## @file SeqADT.py
#  @author Henry Madej
#  @brief Provides the Abstract Data Type (ADT) for representing Sequences
#  @date 06/01/2018


## @brief An abstract data type that represents a Sequence
class SeqT():
  ## @brief SeqT constructor
  #  @details Initializes a SeqT object with the empty sequence
  def __init__(self):
    self.seq = []

  ## @brief add, adds values to/within existing sequence 
  #         or immediately after the last entry in the existing sequence.
  #  @param i The index at which v will be inserted, if i >= length of the 
  #         sequence v will be inserted at the end of the sequence
  #  @param v The real number to be inserted into the sequence
  def add(self, i, v):
    self.seq.insert(i, v)

  ## @brief rm, deletes a value within a sequence at index i
  #  @param i deletes the value at index i shrinking the length of the sequence
  #         by 1
  def rm(self, i):
    del self.seq[i]

  ## @brief set, sets the value at index i in the sequence to value v
  #  @param i the index of the value to be mutated
  #  @param v the new value of the index i of the sequence
  def set(self, i, v):
    self.seq[i] = v
  
  ## @brief get, gets the value of the sequence at index i
  #  @param i the index of the value to be retrieved from the sequence
  #  @return the value of the sequence at index i
  def get(self, i):
    return self.seq[i]

  ## @brief size, returns the size of the sequence
  #  @return the current size of the sequence
  def size(self):
    return len(self.seq)

  ## @brief indexInSeq, returns the index of the value v such that
  #         sequence[i] <= v <= sequence[i+1] if such a value exists otherwise
  #         returns -1
  #  @param v the value to be checked if in sequence
  #  @return the index of the value if it is in the sequence and satisfies
  #           sequence[i] <= v <= sequence[i+1]
  def indexInSeq(self, v):
    index = -1
    for i in range(self.size()-1):
      if self.seq[i] <= v and v <= self.seq[i+1]:
        index = i
        break;
    return index
